# -*- coding: utf-8 -*-

from PythonQt import QtCore,QtGui
from gui import widgetFromUi,mainWindow
from RoadCentre import kernel,plugins
import os
from webbrowser import open as sys_open
from settings import userSettings
from subprocess import Popen
from platform import system as get_platform_name
from threading import Thread
from urllib import parse as url_parser
import binascii

def settingsBool(key):
    return str(userSettings.value(key)).lower()=='true'

def configureLight(light):
    if not light:
        return False
    from RoadCentre import project
    def light_prop(pname, def_val = None):
        pdata = light.getCustomProperty(pname)
        return def_val if len(pdata)==0 else pdata[0]
    use_server = settingsBool("ssh/%d/use_server"%light.id)
    default_server = 'ripas@'+project.connection.remoteHost
    server_host = light_prop('server_host',default_server)
    if server_host is not None and '@' not in server_host:
        server_host = None
    server_port = int(light_prop('server_port', 22))
    controller_host = light_prop('controller_host')
    if controller_host is not None and '@' not in controller_host:
        controller_host = None

    default_controller_ssh_port = int(userSettings.value('ssh/default_controller_ssh_port',22))
    default_controller_web_port = int(userSettings.value('ssh/default_controller_web_port',80))
    default_controller_jrpc_port = int(userSettings.value('ssh/default_controller_jrpc_port',15557))
    default_controller_user = userSettings.value('ssh/default_controller_user','')
    controller_ssh_port = int(light_prop('controller_ssh_port', default_controller_ssh_port))
    controller_web_port = int(light_prop('controller_web_port', default_controller_web_port))
    controller_jrpc_port = int(light_prop('controller_jrpc_port', default_controller_jrpc_port))
    use_modem = settingsBool("ssh/%d/use_modem"%light.id)
    modem_host = light_prop('modem_host')
    if modem_host is not None and '@' not in modem_host:
        modem_host = None
    default_modem_ssh_port = int(userSettings.value('ssh/default_modem_ssh_port',22))
    default_modem_web_port = int(userSettings.value('ssh/default_modem_web_port',80))
    default_modem_user = userSettings.value('ssh/default_modem_user','')
    modem_ssh_port = int(light_prop('modem_ssh_port', default_modem_ssh_port))
    modem_web_port = int(light_prop('modem_web_port', default_modem_web_port))
    direct_controller_access = False
    dlg = widgetFromUi('ssh/ssh_config.ui')
    for item in dlg.findChildren('QWidget'):
        dlg.setProperty(item.objectName, item)
    try:
        server_conn = server_host.split('@')
        server_conn[1] is None
    except (AttributeError, IndexError):
        server_conn = ['', '']
    dlg.server_group.checked = use_server is None or use_server
    dlg.server_host.text = server_conn[1]
    dlg.server_port.value = server_port
    dlg.server_username.text = server_conn[0]

    dlg.modem_group.checked = use_modem
    try:
        modem_conn = modem_host.split('@')
        modem_conn[1] is None
    except (AttributeError, IndexError):
        modem_conn = ['', '']
    dlg.modem_host.text = modem_conn[1]
    dlg.modem_ssh_port.value = modem_ssh_port
    dlg.modem_web_port.value = modem_web_port
    dlg.modem_username.text = modem_conn[0]
    if len(dlg.modem_username.text) == 0:
        dlg.modem_username.text = default_modem_user

    try:
        controller_conn = controller_host.split('@')
        controller_conn[1] is None
    except (AttributeError, IndexError):
        controller_conn = ['', '']
    dlg.controller_ssh_port.value = controller_ssh_port
    dlg.controller_web_port.value = controller_web_port
    dlg.controller_jrpc_port.value = controller_jrpc_port
    dlg.controller_host.text = controller_conn[1]
    dlg.controller_username.text = controller_conn[0]
    if len(dlg.controller_username.text) == 0:
        dlg.controller_username.text = default_controller_user

    if getattr(dlg, 'exec')() != QtGui.QDialog.Accepted:
        return False

    use_server = dlg.server_group.checked
    server_host = dlg.server_username.text.strip() + '@' + dlg.server_host.text.strip()
    server_port = dlg.server_port.value

    if server_host != default_server:
        light.setCustomProperty('server_host', server_host)
    if server_port != 22:
        light.setCustomProperty('server_port', server_port)
    userSettings.setValue("ssh/%d/use_server"%light.id,use_server)

    use_modem = dlg.modem_group.checked
    modem_host = dlg.modem_username.text.strip() + '@' + dlg.modem_host.text.strip()
    modem_ssh_port = dlg.modem_ssh_port.value
    modem_web_port = dlg.modem_web_port.value
    userSettings.setValue("ssh/%d/use_modem"%light.id,use_modem)
    light.setCustomProperty('modem_host', modem_host)
    light.setCustomProperty('modem_ssh_port', modem_ssh_port)
    light.setCustomProperty('modem_web_port', modem_web_port)

    controller_host = dlg.controller_username.text.strip() + '@' + dlg.controller_host.text.strip()
    controller_ssh_port = dlg.controller_ssh_port.value
    controller_web_port = dlg.controller_web_port.value
    controller_jrpc_port = dlg.controller_jrpc_port.value
    light.setCustomProperty('controller_host', controller_host)
    light.setCustomProperty('controller_ssh_port', controller_ssh_port)
    light.setCustomProperty('controller_web_port', controller_web_port)
    light.setCustomProperty('controller_jrpc_port', controller_jrpc_port)
    project.changed = True
    return True

def configureDefaults():
    dlg = widgetFromUi('ssh/ssh_defaults.ui')
    for item in dlg.findChildren('QWidget'):
        dlg.setProperty(item.objectName, item)

    dlg.modem_ssh_port.value = int(userSettings.value('ssh/default_modem_ssh_port',22))
    dlg.modem_web_port.value = int(userSettings.value('ssh/default_modem_web_port',80))
    dlg.modem_username.text = userSettings.value('ssh/default_modem_user','')

    dlg.controller_ssh_port.value = int(userSettings.value('ssh/default_controller_ssh_port',22))
    dlg.controller_web_port.value = int(userSettings.value('ssh/default_controller_web_port',80))
    dlg.controller_jrpc_port.value = int(userSettings.value('ssh/default_controller_jrpc_port',15557))
    dlg.controller_username.text = userSettings.value('ssh/default_controller_user','')


    def get_modem_pass():
        if kernel.wallet.interactiveUnlock(0):
            psw = kernel.wallet.getPassword('__default_modem_login__')
            psw = QtGui.QInputDialog.getText(0,u'Пароль по-умолчанию для модемов',u'пароль',QtGui.QLineEdit.Password,psw)
            if len(psw)>0:
                kernel.wallet.setPassword('__default_modem_login__',psw)

    def get_controller_pass():
        if kernel.wallet.interactiveUnlock(0):
            psw = kernel.wallet.getPassword('__default_controller_login__')
            psw = QtGui.QInputDialog.getText(0,u'Пароль по-умолчанию для контроллеров',u'пароль',QtGui.QLineEdit.Password,psw)
            if len(psw)>0:
                kernel.wallet.setPassword('__default_controller_login__',psw)

    dlg.set_modem_password.connect('clicked()',get_modem_pass)
    dlg.set_controller_password.connect('clicked()',get_controller_pass)

    if getattr(dlg, 'exec')() != QtGui.QDialog.Accepted:
        return False

    userSettings.setValue('ssh/default_modem_ssh_port',dlg.modem_ssh_port.value)
    userSettings.setValue('ssh/default_modem_web_port',dlg.modem_web_port.value)
    userSettings.setValue('ssh/default_modem_user',dlg.modem_username.text)
    userSettings.setValue('ssh/default_controller_ssh_port',dlg.controller_ssh_port.value)
    userSettings.setValue('ssh/default_controller_web_port',dlg.controller_web_port.value)
    userSettings.setValue('ssh/default_controller_jrpc_port',dlg.controller_jrpc_port.value)
    userSettings.setValue('ssh/default_controller_user',dlg.controller_username.text)
    return True

def username(host_str): return host_str.split('@')[0]
def hostname(host_str): return host_str.split('@')[1]
def selectedLights():
    for sel in kernel.selection():
        if sel.inherits('Spectr::Light'):
            yield sel
def setupSSH():
    for sel in selectedLights():
        configureLight(sel)
        break

class SSHSequence:
    def __init__(self,light,title,caller_func=None):
        from RoadCentre import project
        self.ready = False
        self.firstCommand = None
        self.lastCommand = None
        self.server_tun = None
        self.modem_tun = None
        if not self.checkLightConfig(light) and not configureLight(light):
            return
        if not self.checkLightConfig(light):
            return
        use_server = settingsBool("ssh/%d/use_server"%light.id)
        if use_server:
            default_server = 'ripas@'+project.connection.remoteHost
            self.server_host = self.light_prop(light,'server_host',default_server)
            server_port = int(self.light_prop(light,'server_port', 22))
        use_modem = settingsBool("ssh/%d/use_modem"%light.id)
        self.modem_host = self.light_prop(light,'modem_host')
        modem_ssh_port = int(self.light_prop(light,'modem_ssh_port', 22))
        self.modem_web_port = int(self.light_prop(light,'modem_web_port', 80))
        self.controller_host = self.light_prop(light,'controller_host')
        self.controller_ssh_port = int(self.light_prop(light,'controller_ssh_port', 22))
        self.controller_web_port = int(self.light_prop(light,'controller_web_port', 8080))
        self.controller_jrpc_port = int(self.light_prop(light,'controller_jrpc_port', 15557))
        self.ssh_window = plugins.Service.create_ssh_log_window(title)
        self.ssh_window.connect('destroyed()',self.cleanup)

        b = self.ssh_window.findChild('QTextBrowser')
        if b:
            def open_url(url):
                scheme = url.scheme()
                if scheme == 'file':
                    if url.path().endswith('.json'):
                        if hasattr(plugins,'concon'):
                            plugins.concon.openConfig(url.path())
                    else:
                        sys_open(url.path())
                elif scheme == 'ssh':
                    sys_name = get_platform_name().lower()
                    if sys_name == 'windows':
                        putty_location_changed = False
                        putty_exe = userSettings.value('ssh/putty')
                        if putty_exe is None or not os.path.exists(putty_exe):
                            for path in os.environ.get('PATH','').split(os.pathsep):
                                f_name = os.path.join(path,'putty.exe')
                                if os.path.exists(f_name):
                                    putty_exe = f_name
                                    putty_location_changed = True
                                    break
                        if putty_exe is None:
                            putty_exe = QtGui.QFileDialog.getOpenFileName(mainWindow,u'Укажите путь к putty.exe','','putty.exe (putty.exe)')
                            putty_location_changed = True
                        if os.path.exists(putty_exe):
                            if putty_location_changed:
                                userSettings.setValue('ssh/putty',putty_exe)
                            Popen([putty_exe,"-ssh",url.userName()+"@"+url.host(),"-P",str(url.port())])
                    elif sys_name == 'linux':
                        Popen(["x-terminal-emulator","-e","ssh",url.userName()+"@"+url.host(),
                        "-o","StrictHostKeyChecking=no","-o","UserKnownHostsFile=/dev/null","-p",str(url.port())])
                elif scheme == 'cmd':
                    Popen(binascii.unhexlify(url.path()[1:].encode()).decode().split(' '))
            b.connect('anchorClicked(const QUrl &)',open_url)
        if use_server:
            if use_modem:
                self.server_tun = plugins.Service.create_ssh_tunnel('//%s:%d'%(self.server_host,server_port),hostname(self.modem_host),modem_ssh_port)
                self.modem_host=username(self.modem_host)+'@localhost'
            else:
                self.server_tun = plugins.Service.create_ssh_tunnel('//%s:%d'%(self.server_host,server_port),hostname(self.controller_host),self.controller_ssh_port)
                self.controller_host = username(self.controller_host)+'@localhost'
        if use_modem:
            self.modem_tun = plugins.Service.create_ssh_tunnel('//%s:%d'%(self.modem_host,modem_ssh_port),hostname(self.controller_host),self.controller_ssh_port)
            self.modem_tun.setDefaultLogin('__default_modem_login__')
            self.controller_host=username(self.controller_host)+'@localhost'
        if self.server_tun:
            self.server_tun.connect('message(QString,int)',self.ssh_window,'addMessage(QString,int)',QtCore.Qt.QueuedConnection)
        if self.modem_tun:
            self.modem_tun.connect('message(QString,int)',self.ssh_window,'addMessage(QString,int)',QtCore.Qt.QueuedConnection)
        if self.modem_tun:
            if self.server_tun:
                self.server_tun.connect('ready(QString,int,int)',self.modem_tun,'start(QString,int,int)',QtCore.Qt.QueuedConnection)
        if caller_func is not None:
            def repeate():
                self.ssh_window.connect('destroyed()',caller_func)
                self.ssh_window.parent().close()
            self.ssh_window.connect('repeate()',repeate)
        else:
            self.ssh_window.findChild('QPushButton').hide()
        self.ssh_window.connect('setup()',setupSSH)
        self.ready = True

    def cleanup(self):
        if self.lastCommand:
            self.lastCommand.deleteLater()
        if self.firstCommand and self.firstCommand != self.lastCommand:
            self.firstCommand.deleteLater()
        if self.modem_tun:
            self.modem_tun.deleteLater()
        if self.server_tun:
            self.server_tun.deleteLater()
        self.firstCommand = None
        self.lastCommand = None
        self.modem_tun = None
        self.server_tun = None

    def stop(self):
        if self.lastCommand:
            self.lastCommand.stop()
        if self.firstCommand and self.firstCommand != self.lastCommand:
            self.firstCommand.stop()
        if self.modem_tun:
            self.modem_tun.stop()
        if self.server_tun:
            self.server_tun.stop()

    def __del__(self):
        self.cleanup()

    def checkLightConfig(self,light):
        from RoadCentre import project
        controller_host = self.light_prop(light,'controller_host')
        if controller_host is None or '@' not in controller_host:
            return False
        use_modem = userSettings.value("ssh/%d/use_modem"%light.id)
        if use_modem is None:
            return False
        use_modem = str(use_modem).lower()=='true'
        modem_host = self.light_prop(light,'modem_host')
        modem_ssh_port = int(self.light_prop(light,'modem_ssh_port', 22))
        if use_modem:
            if modem_host is None or '@' not in modem_host:
                return False
        use_server = userSettings.value("ssh/%d/use_server"%light.id)
        if use_server is None:
            return False
        use_server = str(use_server).lower()=='true'
        if use_server:
            default_server = 'ripas@'+project.connection.remoteHost
            server_host = self.light_prop(light,'server_host',default_server)
            server_port = int(self.light_prop(light,'server_port', 22))
            if server_host is None or '@' not in server_host:
                return False
        return True

    def light_prop(self,light,pname, def_val = None):
        pdata = light.getCustomProperty(pname)
        return def_val if len(pdata)==0 else pdata[0]

    def controller_ssh_url(self):
        return '//%s:%d'%(self.controller_host,self.controller_ssh_port)

    def addCommand(self, ssh_cmd):
        if not self.ready:
            return
        if self.lastCommand:
            self.lastCommand.addSubcommand(ssh_cmd)
        else:
            self.firstCommand = ssh_cmd
        self.lastCommand = ssh_cmd
        ssh_cmd.connect('message(QString,int)',self.ssh_window,'addMessage(QString,int)',QtCore.Qt.QueuedConnection)

    def startCommand(self, ssh_cmd):
        self.addCommand(ssh_cmd)
        self.start()

    def start(self):
        if not self.ready:
            return
        def add_ssh_msg(ssh_addr,title):
            self.ssh_window.addHtmlMessage(u'Открыть ssh консоль на <a href="ssh://%s">%s</a><br>'%(ssh_addr,title))
        def add_link_to_controller_ssh(port):
            ssh_addr = "%s@localhost:%d"%(username(self.controller_host),port)
            add_ssh_msg(ssh_addr,u'контроллер')
        if self.modem_tun:
            self.modem_tun.connect('portReady(int)',add_link_to_controller_ssh)
            if self.server_tun:
                def add_link_to_modem_ssh(port):
                    ssh_addr = "%s@localhost:%d"%(username(self.modem_host),port)
                    add_ssh_msg(ssh_addr,u'модем')
                self.server_tun.connect('portReady(int)',add_link_to_modem_ssh)
        elif self.server_tun:
            self.server_tun.connect('portReady(int)',add_link_to_controller_ssh)
        if self.firstCommand:
            if self.modem_tun:
                self.modem_tun.connect('ready(QString,int,int)',self.firstCommand,'start(QString,int,int)',QtCore.Qt.QueuedConnection)
                if self.server_tun:
                    self.server_tun.start()
                else:
                    self.modem_tun.start()
            else:
                if self.server_tun:
                    self.server_tun.connect('ready(QString,int,int)',self.firstCommand,'start(QString,int,int)',QtCore.Qt.QueuedConnection)
                    self.server_tun.start()
                else:
                    self.firstCommand.start()
        else:
            if not self.server_tun and not self.modem_tun:
                ssh_addr = "%s@%s:%d"%(username(self.controller_host),hostname(self.controller_host),self.controller_ssh_port)
                add_ssh_msg(ssh_addr,u'контроллер')
                return
            if self.modem_tun:
                if self.server_tun:
                    self.server_tun.start()
                else:
                    self.modem_tun.start()
            else:
                self.server_tun.start()


class SSHSubmenu(QtGui.QMenu):
    def __init__(self,name,parent):
        QtGui.QMenu.__init__(self,name,parent)
        a = self.addAction(u'Считать конфигурацию')
        a.connect('triggered()', self.downloadConfig)
        a = self.addAction(u'Загрузить журнал')
        a.connect('triggered()', self.downloadJournal)
        a = self.addAction(u'Смотреть журнал в реальном времени')
        a.connect('triggered()', self.viewJournalRealtime)
        a = self.addAction(u'Обновить конфигурацию ...')
        a.connect('triggered()', self.uploadConfig)
        a = self.addAction(u'Выполнить обновление ПО ...')
        a.connect('triggered()', self.updateSoftware)
        a = self.addAction(u'Выполнить обновление ПО с прокси...')
        a.connect('triggered()', self.updateSoftwareWithProxy)
        a = self.addAction(u'Выполнить обновление ПО периферии...')
        a.connect('triggered()', self.updatePeripherySoftware)
        a = self.addAction(u'Выполнить комплексное обновление ...')
        a.connect('triggered()', self.complexUpgrade)
        a = self.addAction(u'Открыть туннель на контроллер ...')
        a.connect('triggered()', self.openTunOnController)
        a = self.addAction(u'Выполнить команду ...')
        a.connect('triggered()', self.execCommand)
        a = self.addAction(u'Выполнить команду с прокси...')
        a.connect('triggered()', self.execCommandWithProxy)
        a = self.addAction(u'Открыть веб-интерфейс для модема')
        a.connect('triggered()', self.startBrowserForModem)
        a = self.addAction(u'Открыть веб-интерфейс для контроллера')
        a.connect('triggered()', self.startBrowserForController)
        self.qtest_panel_exe = None
        path_list = [QtCore.QDir.toNativeSeparators(QtCore.QCoreApplication.applicationDirPath())]
        path_list += os.environ.get('PATH','').split(os.pathsep)
        exe_name = 'qtestpanel'
        if get_platform_name().lower() == 'windows':
            exe_name += '.exe'
        for path in path_list:
            f_name = os.path.join(path,exe_name)
            if os.path.exists(f_name):
                self.qtest_panel_exe = f_name
                break
        if self.qtest_panel_exe is not None:
            a = self.addAction(u'Открыть служебную панель для контроллера')
            a.connect('triggered()', self.startTestPanel)
        a = self.addAction(u'Настроить ...')
        a.connect('triggered()', setupSSH)
        self.sudo_pattern = 'sudo-auto-pass'


    def beforeMenu(self, menu, connected):
        if True or len(connected) == 0:
            menu.menuMain.addMenu(self)
        self.old_beforeMenu(menu,connected)

    def downloadConfig(self):
        from RoadCentre import project
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Загрузка конфигурации через SSH для %d'%light.id,self.downloadConfig)
            if not ssh_seq.ready:
                continue
            ssh_transfer = plugins.Service.ssh_transfer(ssh_seq.controller_ssh_url())
            ssh_transfer.setDefaultLogin('__default_controller_login__')
            conf_dir = 'conf'
            d = QtCore.QDir(project.baseDir)
            if d.mkpath(conf_dir):
                d.cd(conf_dir)
                conf_dir = QtCore.QDir.toNativeSeparators(d.absolutePath())
            local_file_name = os.path.join(conf_dir,'dat%.5d.json'%int(light.id))
            ssh_transfer.download('/home/spectr/qspectr/config.json',local_file_name)
            def add_link_to_file(r,l):
                ssh_seq.ssh_window.addHtmlMessage(u'Конфигурация <a href="file://%s">%s</a> загружена<br>'%(l.replace('\\','/'),os.path.split(l)[1]))
            ssh_transfer.connect('fileDownloaded(const QString &,const QString &)',add_link_to_file)
            ssh_seq.startCommand(ssh_transfer)

    def downloadJournal(self):
        from RoadCentre import project
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Загрузка журнала через SSH для %d'%light.id,self.downloadJournal)
            if not ssh_seq.ready:
                continue
            ssh_transfer = plugins.Service.ssh_transfer(ssh_seq.controller_ssh_url())
            ssh_transfer.setDefaultLogin('__default_controller_login__')
            journals_dir = 'journals'
            d = QtCore.QDir(project.baseDir)
            if d.mkpath(journals_dir):
                d.cd(journals_dir)
                journals_dir = QtCore.QDir.toNativeSeparators(d.absolutePath())
            local_file_name = os.path.join(journals_dir,'Jrn%.5d.txt'%int(light.id))
            log_file = '/home/spectr/qspectr/log/qspectr.log'
            if light.type=='МФУ':
                log_file = '/tmp/log/qspectr/qspectr.log'
            ssh_transfer.download(log_file,local_file_name)
            def add_link_to_file(r,l):
                ssh_seq.ssh_window.addHtmlMessage(u'Журнал <a href="file://%s">%s</a> загружен<br>'%(l,os.path.split(l)[1]))
            ssh_transfer.connect('fileDownloaded(const QString &,const QString &)',add_link_to_file)
            ssh_seq.startCommand(ssh_transfer)


    def uploadConfig(self):
        from RoadCentre import project
        dlg = widgetFromUi('ssh/ssh_upload.ui')
        dlg.restart_qspectr.checked=int(userSettings.value('ssh/restart_qspectr_after_upload',1))==1
        conf_dir = ''
        def select_config():
            dlg.file_name.text=QtGui.QFileDialog.getOpenFileName(mainWindow,u'Файл конфигурации',conf_dir,u'Файлы конфигурации (*.cnf *.json);;Все файлы (*.*)')
        dlg.select_file.connect('clicked()',select_config)
        for item in dlg.findChildren('QWidget'):
            dlg.setProperty(item.objectName, item)
        for light in selectedLights():
            conf_dir = 'conf'
            d = QtCore.QDir(project.baseDir)
            if d.mkpath(conf_dir):
                d.cd(conf_dir)
                conf_dir = QtCore.QDir.toNativeSeparators(d.absolutePath())
            local_file_name = os.path.join(conf_dir,'dat%.5d.json'%int(light.id))
            if os.path.exists(local_file_name):
                dlg.file_name.text = local_file_name
            if getattr(dlg, 'exec')() != QtGui.QDialog.Accepted:
                return
            userSettings.setValue('ssh/restart_qspectr_after_upload',int(dlg.restart_qspectr.checked))
            local_file_name = dlg.file_name.text
            if len(local_file_name) == 0:
                continue
            ssh_seq = SSHSequence(light,u'Загрузка конфигурации в контроллер %d через SSH'%light.id,self.uploadConfig)
            if not ssh_seq.ready:
                continue
            ssh_transfer = plugins.Service.ssh_transfer(ssh_seq.controller_ssh_url())
            ssh_transfer.setDefaultLogin('__default_controller_login__')
            ssh_transfer.upload(local_file_name,'/home/spectr/qspectr/config_new.json')
            def add_link_to_file(l,r):
                ssh_seq.ssh_window.addHtmlMessage(u'Конфигурация <a href="file://%s">%s</a> загружена<br>'%(l,os.path.split(l)[1]))
            ssh_transfer.connect('fileUploaded(const QString &,const QString &)',add_link_to_file)
            ssh_seq.addCommand(ssh_transfer)
            cmd_str = 'cp -f /home/spectr/qspectr/config.json /home/spectr/qspectr/config.bak'
            cmd_str += ' && mv -f /home/spectr/qspectr/config_new.json /home/spectr/qspectr/config.json'
            cmd_str += ' && sudo -p %s chown spectr:spectr /home/spectr/qspectr/config.json /home/spectr/qspectr/config.bak' % self.sudo_pattern
            if dlg.restart_qspectr.checked:
                cmd_str += ' && sudo -p %s restart qspectr' % self.sudo_pattern
            ssh_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),cmd_str)
            ssh_cmd.setDefaultLogin('__default_controller_login__')
            ssh_cmd.setSudoPattern(self.sudo_pattern)
            ssh_seq.ssh_window.connect('inputText(QString)',ssh_cmd.sendTextToChannel)
            ssh_seq.startCommand(ssh_cmd)


    def viewJournalRealtime(self):
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Просмотр журнала в реальном времени через SSH для %d'%light.id,self.viewJournalRealtime)
            if not ssh_seq.ready:
                continue
            log_file = '/home/spectr/qspectr/log/qspectr.log'
            if light.type=='МФУ':
                log_file = '/tmp/log/qspectr/qspectr.log'
            ssh_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),'tail -f %s 2>/dev/null' % log_file)
            ssh_cmd.setDefaultLogin('__default_controller_login__')
            ssh_seq.startCommand(ssh_cmd)


    def updateSoftware(self):
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Обновление ПО контроллера %d'%light.id,self.updateSoftware)
            if not ssh_seq.ready:
                continue
            cmd_str='''
            echo ========== update ===========
            sudo -S -p %(sp)s apt update | tee
            if [ ${PIPESTATUS[0]} -ne 0 ]; then
                echo ========== reconfigure ===========
                sudo -S -p %(sp)s dpkg --configure -a | tee
                [ ${PIPESTATUS[0]} -ne 0 ] && echo reconfigure failed && exit 1
            fi
            for r in `seq 20`; do
            echo ========== upgrade ===========
            sudo -S -p %(sp)s apt -y upgrade | tee
            [ ${PIPESTATUS[0]} -eq 0 ] && echo upgrade done && exit 0
            echo ========== fixing ===========
            sudo -S -p %(sp)s apt-get -f -y install | tee
            [ ${PIPESTATUS[0]} -ne 0 ] && echo fixing failed && exit 1
            done
            echo upgrade failed
            ''' % {'sp':self.sudo_pattern}
            ssh_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),cmd_str)
            ssh_cmd.setDefaultLogin('__default_controller_login__')
            ssh_cmd.setShowCommand(False)
            ssh_cmd.setSudoPattern(self.sudo_pattern)
            ssh_seq.ssh_window.connect('inputText(QString)',ssh_cmd.sendTextToChannel)
            ssh_seq.startCommand(ssh_cmd)


    def updateSoftwareWithProxy(self):
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Обновление ПО контроллера %d'%light.id,self.updateSoftwareWithProxy)
            if not ssh_seq.ready:
                continue
            rev_tun = plugins.Service.create_ssh_reverse_tunnel(ssh_seq.controller_ssh_url(),'localhost',0)
            rev_tun.setDefaultLogin('__default_controller_login__')
            ssh_seq.addCommand(rev_tun)
            ssh_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),'')
            ssh_cmd.setShowCommand(False)
            ssh_cmd.setDefaultLogin('__default_controller_login__')
            ssh_seq.lastCommand = ssh_cmd
            proxy = None
            def create_proxy(remote_port):
                proxy = plugins.Service.create_proxy()
                proxy.setParent(rev_tun)
                rev_tun.portReady(proxy.port())
                cmd_str='''
                export http_proxy=http://127.0.0.1:%(rp)d
                export https_proxy=http://127.0.0.1:%(rp)d
                echo ========== update ===========
                sudo -E -S -p %(sp)s apt-get update | tee
                if [ ${PIPESTATUS[0]} -ne 0 ]; then
                    echo ========== reconfigure ===========
                    sudo -E -S -p %(sp)s dpkg --configure -a | tee
                    [ ${PIPESTATUS[0]} -ne 0 ] && echo reconfigure failed && exit 1
                fi
                for r in `seq 20`; do
                echo ========== upgrade ===========
                sudo -E -S -p %(sp)s apt-get -y upgrade | tee
                [ ${PIPESTATUS[0]} -eq 0 ] && echo upgrade done && exit 0
                echo ========== fixing ===========
                sudo -E -S -p %(sp)s apt-get -f -y install | tee
                [ ${PIPESTATUS[0]} -ne 0 ] && echo fixing failed && exit 1
                done
                echo upgrade failed
                ''' % {'sp':self.sudo_pattern, 'rp':remote_port}
                ssh_cmd.setCommand(cmd_str)
                ssh_cmd.start()
            ssh_cmd.connect('message(QString,int)',ssh_seq.ssh_window,'addMessage(QString,int)',QtCore.Qt.QueuedConnection)
            ssh_seq.ssh_window.connect('inputText(QString)',ssh_cmd.sendTextToChannel)
            ssh_cmd.setSudoPattern(self.sudo_pattern)
            rev_tun.connect('remotePortReady(int)',create_proxy)
            if ssh_seq.modem_tun:
                ssh_seq.modem_tun.connect('ready(QString,int,int)',ssh_cmd,'setAlias(QString,int,int)',QtCore.Qt.QueuedConnection)
            elif ssh_seq.server_tun:
                ssh_seq.server_tun.connect('ready(QString,int,int)',ssh_cmd,'setAlias(QString,int,int)',QtCore.Qt.QueuedConnection)
            ssh_seq.start()

    def updatePeripherySoftware(self):
        dlg = widgetFromUi('ssh/ssh_firmwares.ui')
        for item in dlg.findChildren('QWidget'):
            dlg.setProperty(item.objectName, item)
        old_fw_dir = fw_dir = userSettings.value('ssh/last_fw_dir','')
        def add_fw_file():
            fw_file_name=QtGui.QFileDialog.getOpenFileName(dlg,u'Файл прошивки',fw_dir,u'Файлы прошивок (*.bin);;Все файлы (*.*)')
            if len(fw_file_name):
                dlg.fw_list.addItem(fw_file_name)
        def add_fw_from_folder():
            add_fw_from_folder.dir = QtGui.QFileDialog.getExistingDirectory(dlg,u'Каталог с прошивками',old_fw_dir)
            if not len(add_fw_from_folder.dir):
                return
            add_fw_from_folder.executed = True
            for dirpath, dirnames, filenames in os.walk(add_fw_from_folder.dir):
                for f_name in filenames:
                    if os.path.splitext(f_name)[1].lower()=='.bin':
                        dlg.fw_list.addItem(os.path.join(dirpath,f_name))
        add_fw_from_folder.dir = None
        def rm_fw_files():
            selection = dlg.fw_list.selectedItems()
            for sel in selection:
                sel.delete()
        def fw_list_sel_changed():
            dlg.remove_fw.setEnabled(len(dlg.fw_list.selectedItems())>0)
        dlg.remove_fw.setEnabled(False)
        dlg.add_fw.connect('clicked()',add_fw_file)
        dlg.add_from_folder.connect('clicked()',add_fw_from_folder)
        dlg.remove_fw.connect('clicked()',rm_fw_files)
        dlg.fw_list.connect('itemSelectionChanged()',fw_list_sel_changed)
        if getattr(dlg, 'exec')() != QtGui.QDialog.Accepted:
            return False
        if add_fw_from_folder.dir is not None:
            fw_dir = add_fw_from_folder.dir
        fw_files = []
        for i in range(dlg.fw_list.count):
            fw_file = dlg.fw_list.item(i).text()
            if os.path.exists(fw_file):
                if add_fw_from_folder.dir is None:
                    fw_dir = os.path.split(fw_file)[0]
                fw_files.append(fw_file)
        if not len(fw_files):
            return
        if old_fw_dir != fw_dir:
            userSettings.setValue('ssh/last_fw_dir',fw_dir)
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Обновление ПО периферии на контроллере %d'%light.id,self.updatePeripherySoftware)
            if not ssh_seq.ready:
                continue
            ssh_transfer = plugins.Service.ssh_transfer(ssh_seq.controller_ssh_url())
            ssh_transfer.setDefaultLogin('__default_controller_login__')
            remote_fw_dir = '/tmp/firmwares'
            for fw_file in fw_files:
                ssh_transfer.upload(fw_file,remote_fw_dir+'/'+os.path.split(fw_file)[1])
            prepare_cmd_str = 'rm -drf %s; mkdir -p %s' % (remote_fw_dir,remote_fw_dir)
            ssh_prepare_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),prepare_cmd_str)
            ssh_prepare_cmd.setShowCommand(False)
            ssh_prepare_cmd.setDefaultLogin('__default_controller_login__')
            ssh_seq.addCommand(ssh_prepare_cmd)
            ssh_seq.addCommand(ssh_transfer)
            ssh_upgrade_cmd_str = 'sudo stop qspectr\n'
            ssh_upgrade_cmd_str += 'qbusutils autoupdate %s\n' % remote_fw_dir
            ssh_upgrade_cmd_str += 'sudo start qspectr'
            ssh_upgrade_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),ssh_upgrade_cmd_str)
            ssh_upgrade_cmd.setShowCommand(False)
            ssh_upgrade_cmd.setDefaultLogin('__default_controller_login__')
            ssh_seq.addCommand(ssh_upgrade_cmd)
            ssh_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),'rm -drf %s' % remote_fw_dir)
            ssh_cmd.setShowCommand(False)
            ssh_cmd.setDefaultLogin('__default_controller_login__')
            ssh_seq.startCommand(ssh_cmd)

    def complexUpgrade(self):
        pass
        dlg = widgetFromUi('ssh/ssh_full_upgrade.ui')
        for item in dlg.findChildren('QWidget'):
            dlg.setProperty(item.objectName, item)
        #подготовка раздела с прошивками ksk
        old_fw_dir = fw_dir = userSettings.value('ssh/last_fw_dir','')
        def add_fw_file():
            fw_file_name=QtGui.QFileDialog.getOpenFileName(dlg,u'Файл прошивки',fw_dir,u'Файлы прошивок (*.bin);;Все файлы (*.*)')
            if len(fw_file_name):
                dlg.fw_list.addItem(fw_file_name)
        def add_fw_from_folder():
            add_fw_from_folder.dir = QtGui.QFileDialog.getExistingDirectory(dlg,u'Каталог с прошивками',old_fw_dir)
            if not len(add_fw_from_folder.dir):
                return
            add_fw_from_folder.executed = True
            for dirpath, dirnames, filenames in os.walk(add_fw_from_folder.dir):
                for f_name in filenames:
                    if os.path.splitext(f_name)[1].lower()=='.bin':
                        dlg.fw_list.addItem(os.path.join(dirpath,f_name))
        add_fw_from_folder.dir = None
        def rm_fw_files():
            selection = dlg.fw_list.selectedItems()
            for sel in selection:
                sel.delete()
        def fw_list_sel_changed():
            dlg.remove_fw.setEnabled(len(dlg.fw_list.selectedItems())>0)
        dlg.remove_fw.setEnabled(False)
        dlg.add_fw.connect('clicked()',add_fw_file)
        dlg.add_from_folder.connect('clicked()',add_fw_from_folder)
        dlg.remove_fw.connect('clicked()',rm_fw_files)
        dlg.fw_list.connect('itemSelectionChanged()',fw_list_sel_changed)

        #подготовка раздела с загрузкой конфигурации
        dlg.restart_qspectr.checked=int(userSettings.value('ssh/restart_qspectr_after_upload',1))==1
        conf_dir = ''
        def select_config():
            dlg.file_name.text=QtGui.QFileDialog.getOpenFileName(mainWindow,u'Файл конфигурации',conf_dir,u'Файлы конфигурации (*.cnf *.json);;Все файлы (*.*)')
        dlg.select_file.connect('clicked()',select_config)
        for item in dlg.findChildren('QWidget'):
            dlg.setProperty(item.objectName, item)

        dlg.use_proxy.checked = int(userSettings.value('ssh/cu_use_proxy',0))==1
        dlg.upgrade_software.checked = int(userSettings.value('ssh/cu_upgrade_software',0))==1
        dlg.upgrade_firmware.checked = int(userSettings.value('ssh/cu_upgrade_firmware',0))==1
        dlg.upload_config.checked = int(userSettings.value('ssh/cu_upload_config',0))==1
        for fw in userSettings.value('ssh/cu_fw_list',[]):
            dlg.fw_list.addItem(fw)
        if getattr(dlg, 'exec')() != QtGui.QDialog.Accepted:
            return False
        userSettings.setValue('ssh/cu_use_proxy', int(dlg.use_proxy.checked))
        userSettings.setValue('ssh/cu_upgrade_software', int(dlg.upgrade_software.checked))
        userSettings.setValue('ssh/cu_upgrade_firmware', int(dlg.upgrade_firmware.checked))
        userSettings.setValue('ssh/cu_upload_config', int(dlg.upload_config.checked))
        fw_list = []
        for c in range(dlg.fw_list.count):
            fw_list.append(dlg.fw_list.item(c).text())

        userSettings.setValue('ssh/cu_fw_list', fw_list)

        upgrade_cmd_str='''
        echo ========== update ===========
        sudo -S -p %(sp)s apt update | tee
        if [ ${PIPESTATUS[0]} -ne 0 ]; then
            echo ========== reconfigure ===========
            sudo -S -p %(sp)s dpkg --configure -a | tee
            [ ${PIPESTATUS[0]} -ne 0 ] && echo reconfigure failed && exit 1
        fi
        for r in `seq 20`; do
        echo ========== upgrade ===========
        sudo -S -p %(sp)s apt -y upgrade | tee
        [ ${PIPESTATUS[0]} -eq 0 ] && echo upgrade done && exit 0
        echo ========== fixing ===========
        sudo -S -p %(sp)s apt-get -f -y install | tee
        [ ${PIPESTATUS[0]} -ne 0 ] && echo fixing failed && exit 1
        done
        echo upgrade failed
        ''' % {'sp':self.sudo_pattern}

        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Комплексное обновление ПО контроллера %d'%light.id,self.complexUpgrade)
            if not ssh_seq.ready:
                continue
            if dlg.use_proxy.checked:
                rev_tun = plugins.Service.create_ssh_reverse_tunnel(ssh_seq.controller_ssh_url(),'localhost',0)
                rev_tun.setDefaultLogin('__default_controller_login__')
                ssh_seq.addCommand(rev_tun)
                def create_proxy(remote_port):
                    proxy = plugins.Service.create_proxy()
                    proxy.setParent(rev_tun)
                    rev_tun.portReady(proxy.port())
                    if dlg.upgrade_software.checked:
                        proxy_str='''
                        export http_proxy=http://127.0.0.1:%(rp)d
                        export https_proxy=http://127.0.0.1:%(rp)d
                        ''' % {'rp':remote_port}
                        upgrade_cmd.setCommand(proxy_str + upgrade_cmd_str)
                        upgrade_cmd.start()
                rev_tun.connect('remotePortReady(int)',create_proxy)


            upgrade_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),'')
            upgrade_cmd.setShowCommand(False)
            upgrade_cmd.setDefaultLogin('__default_controller_login__')
            upgrade_cmd.setSudoPattern(self.sudo_pattern)

            if dlg.upgrade_software.checked:
                if not dlg.use_proxy.checked:
                    upgrade_cmd.setCommand(upgrade_cmd_str)
                ssh_seq.addCommand(upgrade_cmd)

            while dlg.upgrade_firmware.checked:
                if add_fw_from_folder.dir is not None:
                    fw_dir = add_fw_from_folder.dir
                fw_files = []
                for i in range(dlg.fw_list.count):
                    fw_file = dlg.fw_list.item(i).text()
                    if os.path.exists(fw_file):
                        if add_fw_from_folder.dir is None:
                            fw_dir = os.path.split(fw_file)[0]
                        fw_files.append(fw_file)
                if not len(fw_files):
                    break
                if old_fw_dir != fw_dir:
                    userSettings.setValue('ssh/last_fw_dir',fw_dir)
                ssh_transfer = plugins.Service.ssh_transfer(ssh_seq.controller_ssh_url())
                ssh_transfer.setDefaultLogin('__default_controller_login__')
                remote_fw_dir = '/tmp/firmwares'
                for fw_file in fw_files:
                    ssh_transfer.upload(fw_file,remote_fw_dir+'/'+os.path.split(fw_file)[1])
                prepare_cmd_str = 'rm -drf %s; mkdir -p %s' % (remote_fw_dir,remote_fw_dir)
                ssh_prepare_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),prepare_cmd_str)
                ssh_prepare_cmd.setShowCommand(False)
                ssh_prepare_cmd.setDefaultLogin('__default_controller_login__')
                ssh_seq.addCommand(ssh_prepare_cmd)
                ssh_seq.addCommand(ssh_transfer)
                ssh_upgrade_cmd_str = 'sudo stop qspectr\n'
                ssh_upgrade_cmd_str += 'qbusutils autoupdate %s\n' % remote_fw_dir
                ssh_upgrade_cmd_str += 'sudo start qspectr'
                ssh_upgrade_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),ssh_upgrade_cmd_str)
                ssh_upgrade_cmd.setShowCommand(False)
                ssh_upgrade_cmd.setDefaultLogin('__default_controller_login__')
                ssh_seq.addCommand(ssh_upgrade_cmd)
                ssh_cleanup_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),'rm -drf %s' % remote_fw_dir)
                ssh_cleanup_cmd.setShowCommand(False)
                ssh_cleanup_cmd.setDefaultLogin('__default_controller_login__')
                ssh_seq.addCommand(ssh_cleanup_cmd)
                break

            while dlg.upload_config.checked:
                userSettings.setValue('ssh/restart_qspectr_after_upload',int(dlg.restart_qspectr.checked))
                local_file_name = dlg.file_name.text
                if len(local_file_name) == 0:
                    break
                ssh_transfer = plugins.Service.ssh_transfer(ssh_seq.controller_ssh_url())
                ssh_transfer.setDefaultLogin('__default_controller_login__')
                ssh_transfer.upload(local_file_name,'/home/spectr/qspectr/config_new.json')
                def add_link_to_file(l,r):
                    ssh_seq.ssh_window.addHtmlMessage(u'Конфигурация <a href="file://%s">%s</a> загружена<br>'%(l,os.path.split(l)[1]))
                ssh_transfer.connect('fileUploaded(const QString &,const QString &)',add_link_to_file)
                ssh_seq.addCommand(ssh_transfer)
                update_config_cmd_str = 'cp -f /home/spectr/qspectr/config.json /home/spectr/qspectr/config.bak'
                update_config_cmd_str += ' && mv -f /home/spectr/qspectr/config_new.json /home/spectr/qspectr/config.json'
                update_config_cmd_str += ' && sudo -p %s chown spectr:spectr /home/spectr/qspectr/config.json /home/spectr/qspectr/config.bak' % self.sudo_pattern
                if dlg.restart_qspectr.checked:
                    update_config_cmd_str += ' && sudo -p %s restart qspectr' % self.sudo_pattern
                update_config_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),update_config_cmd_str)
                update_config_cmd.setDefaultLogin('__default_controller_login__')
                update_config_cmd.setSudoPattern(self.sudo_pattern)
                ssh_seq.ssh_window.connect('inputText(QString)',update_config_cmd.sendTextToChannel)
                ssh_seq.addCommand(update_config_cmd)
                break
            ssh_seq.start()

    def execCommandWithProxy(self):
        dlg = widgetFromUi('ssh/ssh_command.ui')
        for item in dlg.findChildren('QWidget'):
            dlg.setProperty(item.objectName, item)
        last_exec_cmd = getattr(self,'last_exec_cmd',None)
        if last_exec_cmd is not None:
            dlg.textEdit.plainText = last_exec_cmd
        if getattr(dlg, 'exec')() != QtGui.QDialog.Accepted:
            return False
        cmd_text = dlg.textEdit.plainText
        if cmd_text is None:
            return
        self.last_exec_cmd = cmd_text
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Выполнение команды на контроллере %d'%light.id,self.execCommandWithProxy)
            if not ssh_seq.ready:
                continue
            rev_tun = plugins.Service.create_ssh_reverse_tunnel(ssh_seq.controller_ssh_url(),'localhost',0)
            rev_tun.setDefaultLogin('__default_controller_login__')
            ssh_seq.addCommand(rev_tun)
            proxy = None
            ssh_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),cmd_text)
            ssh_cmd.setShowCommand(False)
            ssh_cmd.setDefaultLogin('__default_controller_login__')
            ssh_seq.lastCommand = ssh_cmd
            def create_proxy(remote_port):
                proxy = plugins.Service.create_proxy()
                proxy.setParent(rev_tun)
                rev_tun.portReady(proxy.port())
                ssh_cmd.setCommand('export http_proxy=http://127.0.0.1:%(rp)d\nexport https_proxy=http://127.0.0.1:%(rp)d\n'%{'rp':remote_port}+cmd_text)
                ssh_cmd.start()
            ssh_cmd.connect('message(QString,int)',ssh_seq.ssh_window,'addMessage(QString,int)',QtCore.Qt.QueuedConnection)
            ssh_seq.ssh_window.connect('inputText(QString)',ssh_cmd.sendTextToChannel)
            ssh_cmd.setSudoPattern(self.sudo_pattern)
            rev_tun.connect('remotePortReady(int)',create_proxy)
            if ssh_seq.modem_tun:
                ssh_seq.modem_tun.connect('ready(QString,int,int)',ssh_cmd,'setAlias(QString,int,int)',QtCore.Qt.QueuedConnection)
            elif ssh_seq.server_tun:
                ssh_seq.server_tun.connect('ready(QString,int,int)',ssh_cmd,'setAlias(QString,int,int)',QtCore.Qt.QueuedConnection)
            ssh_seq.start()

    def openTunOnController(self):
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'SSH туннель на контроллер для %d'%light.id,self.openTunOnController)
            if not ssh_seq.ready:
                continue
            ssh_seq.start()

    def execCommand(self):
        dlg = widgetFromUi('ssh/ssh_command.ui')
        for item in dlg.findChildren('QWidget'):
            dlg.setProperty(item.objectName, item)
        last_exec_cmd = getattr(self,'last_exec_cmd',None)
        if last_exec_cmd is not None:
            dlg.textEdit.plainText = last_exec_cmd
        if getattr(dlg, 'exec')() != QtGui.QDialog.Accepted:
            return False
        cmd_text = dlg.textEdit.plainText
        if cmd_text is None:
            return
        self.last_exec_cmd = cmd_text
        if not cmd_text.endswith('\n'):
            cmd_text+='\n'
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Выполнение команды на контроллере %d'%light.id,self.execCommand)
            if not ssh_seq.ready:
                continue
            ssh_cmd = plugins.Service.ssh_command(ssh_seq.controller_ssh_url(),cmd_text)
            ssh_cmd.setDefaultLogin('__default_controller_login__')
            ssh_cmd.setSudoPattern(self.sudo_pattern)
            ssh_seq.ssh_window.connect('inputText(QString)',ssh_cmd.sendTextToChannel)
            ssh_seq.startCommand(ssh_cmd)

    def startBrowserForModem(self):
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Веб-интерфейс на модем для контроллера %d'%light.id,self.startBrowserForModem)
            if not ssh_seq.ready:
                continue
            def openModemLink(port):
                url='http://%s:%d' % (hostname(ssh_seq.modem_host), port)
                thread = Thread(target=sys_open, args=(url,))
                thread.start()
                thread.join(0.3)
            if ssh_seq.server_tun:
                ssh_seq.server_tun.setRemotePort(ssh_seq.modem_web_port)
                ssh_seq.server_tun.connect('portReady(int)',openModemLink)
                ssh_seq.server_tun.start()
            else:
                openModemLink(ssh_seq.modem_web_port)

    def startBrowserForController(self):
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Веб-интерфейс на контроллер %d'%light.id,self.startBrowserForController)
            if not ssh_seq.ready:
                continue
            def openControllerLink(port):
                url='http://%s:%d' % (hostname(ssh_seq.controller_host), port)
                thread = Thread(target=sys_open, args=(url,))
                thread.start()
                thread.join(0.3)
            if ssh_seq.modem_tun:
                ssh_seq.modem_tun.setRemotePort(ssh_seq.controller_web_port)
                ssh_seq.modem_tun.connect('portReady(int)',openControllerLink)
                if ssh_seq.server_tun:
                    ssh_seq.server_tun.start()
                else:
                    ssh_seq.modem_tun.start()
            elif ssh_seq.server_tun:
                ssh_seq.server_tun.setRemotePort(ssh_seq.controller_web_port)
                ssh_seq.server_tun.connect('portReady(int)',openControllerLink)
                if ssh_seq.server_tun:
                    ssh_seq.server_tun.start()
            else:
                openControllerLink(ssh_seq.controller_web_port)

    def startTestPanel(self):
        for light in selectedLights():
            ssh_seq = SSHSequence(light,u'Туннель для панели управления контроллером %d'%light.id,self.startTestPanel)
            if not ssh_seq.ready:
                continue
            def openTestPanel(port):
                popen_param = [self.qtest_panel_exe,"--url","tcp://localhost:%d"%port]
                enc_url=binascii.hexlify(' '.join(popen_param).encode()).decode()
                ssh_seq.ssh_window.addHtmlMessage(u'<a href="cmd://cmd/%s">Открыть тестовую панель</a><br>'%enc_url)
                Popen(popen_param)
            controller_tun = plugins.Service.create_ssh_tunnel('//%s:%d'%(ssh_seq.controller_host,ssh_seq.controller_ssh_port),"localhost",ssh_seq.controller_jrpc_port)
            controller_tun.setDefaultLogin('__default_controller_login__')
            controller_tun.connect('portReady(int)',openTestPanel)
            ssh_seq.addCommand(controller_tun)
            ssh_seq.start()
